//
//  CLController.swift
//  Multipeer
//
//  Created by Felix Wehnert on 20.09.16.
//  Copyright © 2016 Felix Wehnert. All rights reserved.
//

import UIKit
import CoreLocation
import CoreBluetooth
import UserNotifications

class CLController: NSObject, CBPeripheralManagerDelegate, CLLocationManagerDelegate {
    
    var region: CLBeaconRegion
    var clLocationManager: CLLocationManager
    var cbManager: CBPeripheralManager?
    
    static let uuid = UUID(uuidString: "B4A462DB-BC44-4488-8E7D-314B0FFBF950")!
    
    override init() {
        
        region = CLBeaconRegion(proximityUUID: CLController.uuid, identifier: "me.wehnert.felix")
        region.notifyEntryStateOnDisplay = true
        region.notifyOnExit = true
        region.notifyOnEntry = true
        
        clLocationManager = CLLocationManager()
        
        super.init()
        
        clLocationManager.delegate = self
        cbManager = CBPeripheralManager(delegate: self, queue: DispatchQueue.main)
        
        
        switch CLLocationManager.authorizationStatus() {
        case .denied, .restricted, .notDetermined:
            clLocationManager.requestAlwaysAuthorization()
        default:
            break;
        }
        
        clLocationManager.startMonitoring(for: region)
        clLocationManager.startRangingBeacons(in: region)
        
    }
    
    func peripheralManagerDidStartAdvertising(_ peripheral: CBPeripheralManager, error: Error?) {
        print("started beacon advertising error: \(error)")
    }
    
    func peripheralManagerDidUpdateState(_ peripheral: CBPeripheralManager) {
        print("CBPeripheralManager updated state: \(peripheral.state.rawValue)")
        
        let dict: [String : Any] = (region.peripheralData(withMeasuredPower: nil) as NSDictionary) as! [String : Any]
        cbManager!.startAdvertising(dict)
        
        
    }
    
    
    
    func locationManager(_ manager: CLLocationManager, didDetermineState state: CLRegionState, for region: CLRegion) {
        switch state {
        case .inside:
            print("inside \(region.identifier)")
        case .outside:
            print("outside \(region.identifier)")
        case .unknown:
            print("unknown \(region.identifier)")
        }
    }
    
    func locationManager(_ manager: CLLocationManager, didEnterRegion region: CLRegion) {
        print("didEnterRegion")
        
        print("Will send notification")
        let content = UNMutableNotificationContent()
        content.title = "Beacon in near"
        content.subtitle = "That's great"
        content.body = "Now, open to chat"
        
        // Deliver the notification in one second.
        let trigger = UNTimeIntervalNotificationTrigger.init(timeInterval: 1, repeats: false)
        let request = UNNotificationRequest.init(identifier: UUID().uuidString, content: content, trigger: trigger)
        
        // Schedule the notification.
        let center = UNUserNotificationCenter.current()
        center.add(request)
        
    }
    
    var i = 0
    func locationManager(_ manager: CLLocationManager, didRangeBeacons beacons: [CLBeacon], in region: CLBeaconRegion) {
        if self.i%3 == 0 {
            print("Beacons: \(beacons.count) | CBisAdvertising: \(self.cbManager!.isAdvertising)")
        }
        self.i += 1
    }
    
    func locationManager(_ manager: CLLocationManager, didExitRegion region: CLRegion) {
        print("didExitRegion")
    }
    
}
