//
//  MPDelegate.swift
//  Multipeer
//
//  Created by Felix Wehnert on 20.09.16.
//  Copyright © 2016 Felix Wehnert. All rights reserved.
//

import Foundation
import MultipeerConnectivity

class MCController: NSObject {
    
    static let service = "flx-peery"
    
    var peerID: MCPeerID
    var mcSession: MCSession
    var mcBrowser: MCNearbyServiceBrowser
    var mcAdvertiser: MCNearbyServiceAdvertiser
    
    var delegate: MCControllerProtocol?
    
    
    init(displayName: String?) {
        
        let randomIdentifier = Int(arc4random_uniform(999))
        self.peerID = MCPeerID(displayName: displayName ?? "Peer#\(randomIdentifier)")
        
        self.mcSession = MCSession(peer: peerID, securityIdentity: nil, encryptionPreference: .required)
        self.mcBrowser = MCNearbyServiceBrowser(peer: peerID, serviceType: MCController.service)
        self.mcAdvertiser = MCNearbyServiceAdvertiser(peer: peerID, discoveryInfo: nil, serviceType: MCController.service)
        
        super.init()
        
        
        self.mcSession.delegate = self
        self.mcBrowser.delegate = self
        self.mcAdvertiser.delegate = self
        
        self.mcBrowser.startBrowsingForPeers()
        self.mcAdvertiser.startAdvertisingPeer()
        
    }
    
    func sendString(string: String) {
        do {
            try self.mcSession.send(string.data(using: String.Encoding.utf8)!, toPeers: self.mcSession.connectedPeers, with: .reliable)
        } catch {
            print(error)
        }
    }
    
    deinit {
        mcSession.disconnect()
    }
}
    // MARK: - MCSessionDelegate
    
extension MCController: MCSessionDelegate {
    
    func session(_ session: MCSession, didReceive data: Data, fromPeer otherPeer: MCPeerID) {
        print("didRecieve Data")
        
        DispatchQueue.main.async {
            self.delegate!.stringRecieved(recievedString: String(data: data, encoding: String.Encoding.utf8)!, fromPeer: otherPeer)
        }
        
    }
    
    func session(_ session: MCSession, didReceive stream: InputStream, withName streamName: String, fromPeer peerID: MCPeerID) {
        print("didRecieve Stream. Bytes Avail: \(stream.hasBytesAvailable)")
    }
    
    func session(_ session: MCSession, peer otherPeer: MCPeerID, didChange state: MCSessionState) {
        
        switch state {
        case .connected:
            print("connected with \(otherPeer)")
            
        case .connecting:
            print("connecting with \(otherPeer)")
        case .notConnected:
            print("not connected with \(otherPeer)")
        }
        
        DispatchQueue.main.async {
            self.delegate!.stateChanged(state: state, peer: otherPeer)
        }
        
    }
    
    func session(_ session: MCSession, didStartReceivingResourceWithName resourceName: String, fromPeer otherPeer: MCPeerID, with progress: Progress) {
        print("didStartReceivingResourceWithName \(resourceName) from \(otherPeer.displayName) \(Int(progress.fractionCompleted*100))%")
    }
    
    func session(_ session: MCSession, didFinishReceivingResourceWithName resourceName: String, fromPeer otherPeer: MCPeerID, at localURL: URL, withError error: Error?) {
        print("didFinishReceivingResourceWithName \(resourceName) from \(otherPeer.displayName)")
    }
}
    //MARK: MCNearbyBrowserDelegate
extension MCController: MCNearbyServiceBrowserDelegate {
    
    func browser(_ browser: MCNearbyServiceBrowser, lostPeer otherPeer: MCPeerID) {
        print("Lost Peer \(otherPeer)")
    }
    
    func browser(_ browser: MCNearbyServiceBrowser, didNotStartBrowsingForPeers error: Error) {
        print("error browsing")
    }
    
    func browser(_ browser: MCNearbyServiceBrowser, foundPeer otherPeer: MCPeerID, withDiscoveryInfo info: [String : String]?) {
        
        print("Found Peer \(otherPeer). Invitation sent.")
        
        browser.invitePeer(otherPeer, to: self.mcSession, withContext: nil, timeout: 30)
        
    }
}
    
    // MARK: MCNearbyAdvertiser
extension MCController: MCNearbyServiceAdvertiserDelegate {
    
    func advertiser(_ advertiser: MCNearbyServiceAdvertiser, didNotStartAdvertisingPeer error: Error) {
        print("error advertising")
    }
    
    func advertiser(_ advertiser: MCNearbyServiceAdvertiser, didReceiveInvitationFromPeer otherPeer: MCPeerID, withContext context: Data?, invitationHandler: @escaping (Bool, MCSession?) -> Void) {
        print("Accept invitation from \(otherPeer)")
        
        invitationHandler(true, self.mcSession)
    }
    
}
