//
//  ViewController.swift
//  Multipeer
//
//  Created by Felix Wehnert on 20.09.16.
//  Copyright © 2016 Felix Wehnert. All rights reserved.
//

import UIKit
import MultipeerConnectivity

class ViewController: UIViewController, MCControllerProtocol, UITextFieldDelegate {

    @IBOutlet weak var peerLabel: UILabel!
    @IBOutlet weak var textView: UITextView!
    @IBOutlet weak var writeField: UITextField!
    
    var mcController: MCController?
    
    @IBOutlet weak var bottomConstraint: NSLayoutConstraint!
    var bottomConstraintDefaultValue: CGFloat?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        mcController = MCController(displayName: UIDevice.current.name)
        mcController!.delegate = self
        
        peerLabel.text = mcController!.peerID.displayName
        writeField.delegate = self
        
        self.bottomConstraintDefaultValue = self.bottomConstraint.constant
        
        NotificationCenter.default.addObserver(self, selector: #selector(ViewController.keyboardHandler(notification:)), name: Notification.Name.UIKeyboardWillShow, object: nil)
        
        NotificationCenter.default.addObserver(self, selector: #selector(ViewController.keyboardHandler(notification:)), name: Notification.Name.UIKeyboardWillHide, object: nil)
        
    }
    
    func keyboardHandler(notification: Notification) {
        let value: NSValue = notification.userInfo![UIKeyboardFrameBeginUserInfoKey] as! NSValue
        let rect = value.cgRectValue
        let y = rect.size.height
        
        switch notification.name {
        case Notification.Name.UIKeyboardWillShow:
            self.bottomConstraint.constant = self.bottomConstraintDefaultValue! + y
        case Notification.Name.UIKeyboardWillHide:
            self.bottomConstraint.constant = self.bottomConstraintDefaultValue!
        default:
            break
        }
        
        UIView.animate(withDuration: 1) {
            self.view.layoutIfNeeded()
        }
        
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.mcController!.sendString(string: textField.text!)
        append("\(mcController!.peerID): "+textField.text!)
        textField.text = ""
        return true
    }
    
    func stateChanged(state: MCSessionState, peer: MCPeerID) {
        var message: String
        switch state {
        case .connected:
            message = "\(peer) joined the conversation"
        case .connecting:
            message = "\(peer) tries to join the conversation"
        case .notConnected:
            message = "\(peer) left or failed to join the conversation"
        }
        
        append(message)
        
        var peerString = "\(mcController!.peerID) (you) with "
        peerString.append(mcController!.mcSession.connectedPeers.map({$0.displayName}).joined(separator: ", "))
        self.peerLabel.text = peerString
        
    }
    
    func stringRecieved(recievedString string: String, fromPeer peer: MCPeerID) {
        append("\(peer): \(string)")
    }
    
    func append(_ text: String) {
        self.textView.text.append(text + "\n")
    }
    

}

